#include <u.h>
#include <libc.h>
#include <bio.h>
#include "dat.h"

/* Default server */
static char *defserver = "wiretap.area.com";

/* Path to agopher binary for starting new instances. */
static char *agopher = "/bin/agopher";

/* Menu Item Structure */
typedef struct {
	char type;
	char *name;
	char *server;
	char *selector;
} Menu;

/* Menu Items */
static int nitems = 0;
static Menu *menu = 0;

void
usage(void)
{
	print("agopher [server [selector]]\n");
	exits("usage");
}

void
additem(char *line)
{
	char *fields[5];
	int n;

	menu = realloc(menu, sizeof(Menu) * (nitems + 1));

	menu[nitems].type = line[0];
	line++;
	n = getfields(line, fields, 5, 0, "\t\n");
	if(n < 4)
		error("Malformed menu line\n");

	menu[nitems].name = strdup(fields[0]);
	menu[nitems].selector = strdup(fields[1]);
	menu[nitems].server = emalloc(4 + strlen(fields[2]) + 1 + strlen(fields[3]) + 1);
	sprint(menu[nitems].server, "tcp!%s!%s", fields[2], fields[3]);
	nitems++;
}

void
gopher(Window *browse, char *server, char *selector, char type)
{
	if (fork() == 0) {
		char stype[2];

		windormant(browse);
		close(browse->event);
		close(browse->ctl);

		stype[0] = type;
		stype[1] = '\0';
		execl(agopher, "agopher", server, selector, stype, 0);
	}
}

void
gopherget(Window *browse, char *server, char *selector, char type)
{
	int connection;
	Biobuf *bconnection;
	char *line, *name;

	connection = dial(server, 0, 0, 0);
	if(connection < 0){
		fprint(2, "Can't dial %s: %r", server);
		return;
	}

	write(connection, selector, strlen(selector));
	write(connection, "\n", 1);

	bconnection = emalloc(sizeof(Biobuf));
	Binit(bconnection, connection, OREAD);

	if(browse->body == nil)
		winopenbody(browse, OWRITE);

	switch (type){
	case '1':
		Bprint(browse->body, "--- %s %s ---\n", server, selector);
	case '0':
		name = emalloc(14 + strlen(agopher) + 1 + strlen(server) + 1 + strlen(selector) + 1);
		sprint(name, "gopher://%s/%s", server, selector);
		winname(browse, name);
		sprint(name, "%s %s %s", agopher, server, selector);
		winsetdump(browse, "/acme", name);
		free(name);

		while(line=Brdline(bconnection, '\n')){
			int len = Blinelen(bconnection);
			if (line[len-2] == '\r') {
				len -= 1;
				line[len-1] = '\n';
			}
			line[len] = '\0';
			if (line[0] == '.' && line[1] == '\n')
				break;

			if(type == '0')
				Bwrite(browse->body, line, len);
			else{
				additem(line);
				Bprint(browse->body, "%d%s\t%s\n", nitems, (line[0] == '1') ? "/" : "", menu[nitems-1].name);
			}
		}
		break;
	default:
		error("Unknown document type %d\n", type);
	}

	Bterm(bconnection);
	close(connection);
	winclosebody(browse);
	winclean(browse);
}

void
watchevent(Window *browse)
{
	Event e, ea, e2, eq;
	int na, i;
	char *s, *t;

	/* mainly from mail.c */
	for(;;){
		wingetevent(browse, &e);
		switch(e.c1){
		default:
		Unknown:
			print("unknown message %c%c\n", e.c1, e.c2);
			break;
	
		case 'E':	/* write to body; can't affect us */
			break;
	
		case 'F':	/* generated by our actions; ignore */
			break;
	
		case 'K':	/* type away; we don't care */
			break;
	
		case 'M':
			switch(e.c2){
			case 'x':
			case 'X':
				if(e.flag & 2)
					wingetevent(browse, &e2);
				if(e.flag & 8){
					wingetevent(browse, &ea);
					na = ea.nb;
					wingetevent(browse, &eq);		/* Don't care about the arg's address. */
				}else
					na = 0;
				s = e.b;
				if((e.flag&2) && e.nb==0)
					s = e2.b;
				if(na){
					t = emalloc(strlen(s)+1+na+1);
					sprint(t, "%s %s", s, ea.b);
					s = t;
				}
				/* We don't yet implement any button 2 commands. */
				if(1)	/* send it back */
					winwriteevent(browse, &e);
				if(na)
					free(s);
				break;
	
			case 'l':
			case 'L':
				// buf = nil;
				eq = e;
				if(e.flag & 2)
					wingetevent(browse, &eq);
				s = eq.b;
				/* If it's too long, it can't really be what we want anyway. */
				/*
				if(eq.q1>eq.q0 && eq.nb==0){
					buf = emalloc((eq.q1-eq.q0)*UTFmax+1);
					winread(browse, eq.q0, eq.q1, buf);
					s = buf;
				}
				*/

				i = atoi(s);
				if (i > 0 && i <= nitems)
					gopher(browse, menu[i-1].server, menu[i-1].selector, menu[i-1].type);
				else
					winwriteevent(browse, &e);
				//free(buf);
				break;
	
			case 'I':	/* modify away; we don't care */
			case 'D':
			case 'd':
			case 'i':
				break;
	
			default:
				goto Unknown;
			}
		}
	}
}

void
main(int argc, char **argv)
{
	Window *browse;
	char *server;
	char *selector = "";
	char type = '1';

	ARGBEGIN{
	default:
		usage();
	}ARGEND

	if (argc) {
		server = netmkaddr(argv[0], "tcp", "gopher");
		if (argv[1]) {
			selector = argv[1];
			if (argv[2])
				type = argv[2][0];
		}
	} else
		server = netmkaddr(defserver, "tcp", "gopher");
	
	browse = newwindow();

	gopherget(browse, server, selector, type);

	if (type == '1')
		watchevent(browse);

	windormant(browse);
	close(browse->event);
	close(browse->ctl);

	free(browse);
	exits(0);
}
